#!/usr/bin/env python2.6
#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
This script is used to strip down useless info from a full database dump
Full dumps can be found at http://wiki.eve-id.net/CCP_Database_Dump_Resources#Conversions
@author: cncfanatics
'''
import sys
sys.path.append("..")

from optparse import OptionParser
from dataFolder import dataFolder
import bz2
import sqlite3
import shutil

usage = "usage: %prog --f=FILE"
parser = OptionParser(usage=usage)
parser.add_option("-f", "--file", help="path to database")
(options, args) = parser.parse_args()

if options.file == None:
    sys.stderr.write("Expecting a path to a database file as command line argument. Run script with --help option for further info.\n")
    sys.exit()

conn = sqlite3.connect(options.file)
#The tables to keep:
tables = ("dgmAttributeCategories", "dgmAttributeTypes", "dgmEffects", "dgmTypeAttributes",
          "dgmTypeEffects", "eveGraphics", "invCategories", "invGroups", "invMarketGroups",
          "invMetaGroups", "invMetaTypes", "invTypes")
indexes = {"dgmAttributeTypes" : ("categoryID", "graphicID", "unitID"),
           "dgmEffects" : ("graphicID",),
           "dgmTypeAttributes" : ("attributeID",),
           "dgmTypeEffects" : ("effectID",),
           "invCategories" : ("graphicID",),
           "invGroups" : ("categoryID", "graphicID"),
           "invMarketGroups" : ("parentGroupID", "graphicID"),
           "invMetaGroups" : ("graphicID",),
           "invMetaTypes" : ("parentTypeID", "metaGroupID"),
           "invTypes" : ("groupID", "graphicID", "raceID", "marketGroupID"),}

print("removing tables")
c = conn.cursor()
c.execute("SELECT name FROM sqlite_master WHERE type = 'table'")
rows = []
for row in c: rows.append(row[0])
for table in rows:
    if not table in tables: c.execute("DROP TABLE %s" % table)    
print("vacuuming")
c.execute("VACUUM;")
print("generating indexes")
for table, columns in indexes.iteritems():
    for col in columns:
        c.execute('CREATE INDEX IF NOT EXISTS "%s_IX_%s" ON "%s" ("%s")' % (table, col, table, col))
print("committing")
conn.commit()
print("closing sql connection")
conn.close()
print("all done")
